package com.bipt.intelligentapplicationorchestrationservice.controller;


import com.bipt.intelligentapplicationorchestrationservice.pojo.OptResult;
import com.bipt.intelligentapplicationorchestrationservice.service.ServiceAPIService;
import com.bipt.intelligentapplicationorchestrationservice.util.NacosServiceUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.models.security.SecurityScheme;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Tag(name ="服务API相关接口")
@RestController
@RequestMapping("/API")
@Slf4j
public class ServiceAPIController {

    @Autowired
    private ServiceAPIService serviceAPIService;

    @Autowired
    private NacosServiceUtil nacosServiceUtil;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @PostMapping("/release")
    @Operation(summary = "结束访问")
    @Transactional
    public OptResult releaseResource(@PathVariable Long modelId) {
        String key = "modelId:" + modelId;
        String modelConfig = (String) redisTemplate.opsForValue().get(key);
        int userMemorySize = parseGpuMemorySize(modelConfig);
        List<String> instanceIps;
        try {
            instanceIps = nacosServiceUtil.getServiceInstances(modelId.toString());
        } catch (Exception e) {
            log.error("获取Nacos实例失败", e);
            return OptResult.error("获取实例失败");
        }
        int memorySize;
        for (String ip : instanceIps) {
            String ipKey = "ip:" + ip;
            Integer nowMemorySizeOBJ = (Integer) redisTemplate.opsForValue().get(ipKey);
            int nowMemorySize = nowMemorySizeOBJ;
            memorySize = nowMemorySize + userMemorySize;
            // 更新IP对应的资源值
            redisTemplate.opsForValue().set(ipKey, memorySize);
            // 设置缓存过期时间（3600秒）
            redisTemplate.expire(ipKey, 3600, TimeUnit.SECONDS);
        }
    
        // 处理等待队列（先来先服务）
        String waitQueueKey = "waitQueue:" + modelId;
        // 取出队列头部的任务（最早加入的）
        Long waitModelId = (Long) redisTemplate.opsForList().leftPop(waitQueueKey);
        if (waitModelId != null) {
            log.info("检测到等待队列任务，尝试调度模型ID: {}", waitModelId);
            return schedule(waitModelId); // 重新调度最早的任务
        } else {
            log.info("等待队列为空，无任务需要处理");
        }

        return OptResult.success("资源释放成功");
    }

    @PostMapping("/request")
    @Operation(summary = "请求调度")
    @Transactional
    public OptResult schedule(@PathVariable Long modelId) {
        // 1. 存储modelConfig到缓存
        String modelConfig = serviceAPIService.getByModelId(modelId);
        int requestMemorySize = parseGpuMemorySize(modelConfig);
        String modelConfigKey = "modelConfig:" + modelId;
        redisTemplate.opsForValue().set(modelConfigKey, modelConfig);
        // 2. 获取Nacos实例IP列表
        List<String> instanceIps;
        try {
            instanceIps = nacosServiceUtil.getServiceInstances(modelId.toString());
        } catch (Exception e) {
            log.error("获取Nacos实例失败", e);
            return OptResult.error("获取实例失败");
        }
        Set<String> gpuKeys = redisTemplate.keys("gpu:*");
        //根据IP列表查找资源
        for (String instanceIp : instanceIps) {
            for (String gpuKey : gpuKeys) {
                String GPUConfig = (String) redisTemplate.opsForValue().get(gpuKey);
                if (GPUConfig != null) {
                    // 分割键值对
                    String[] pairs = GPUConfig.split(",");
                    String ip = null;
                    int memorySize = 0;
                    for (String pair : pairs) {
                        String[] keyValue = pair.split(":", 2);
                        if (keyValue.length == 2) {
                            String key = keyValue[0].trim();
                            String value = keyValue[1].trim();
                            if ("IP".equalsIgnoreCase(key)) {
                                ip = value;
                            } else if ("GPUMemorySize".equalsIgnoreCase(key)) {
                                memorySize = Integer.parseInt(value);
                            }
                        }
                    }
                    // 检查解析出的 IP 是否在 Nacos 实例列表中
                    if (instanceIp.equals(ip)) {
                        log.info("找到 IP {} 对应的 GPU 内存: {} ", ip, memorySize);
                        if (memorySize>=requestMemorySize){
                            int newMemorySize = memorySize - requestMemorySize;
                            String ipKey = "ip:" + ip;
                            redisTemplate.opsForValue().set(ipKey,newMemorySize);
                            //访问请求最大时间为3600s
                            redisTemplate.expire(ipKey, 3600, TimeUnit.SECONDS);
                        }
                        return OptResult.success("资源分配成功，使用ip:" + ip);
                    }else {
                        log.info("资源不足");
                    }
                }
            }
        }
        // 所有实例检查完毕未找到足够资源
        String waitQueueKey = "waitQueue:" + modelId;
        // 改为右插入，保证队列顺序为FIFO（最早的任务在列表头部）
        redisTemplate.opsForList().rightPush(waitQueueKey, modelId); 
        log.info("未找到足够资源，任务 {} 加入等待队列", modelId);
        return OptResult.error("资源不足，等待中");
    }
    /**
     * 从模型配置字符串中解析GPU内存需求
     * @param modelConfig 模型配置字符串，格式如 "GPUMemorySize:8000,version:1"
     * @return 解析到的GPU内存大小（MB），若解析失败返回-1
     */
    private int parseGpuMemorySize(String modelConfig) {
        if (modelConfig == null || modelConfig.isEmpty()) {
            log.error("模型配置为空，无法解析GPU内存需求");
            return -1;
        }
        int requestMemorySize = 0;
        String[] config = modelConfig.split(",");
        for (String pair : config) {
            // 按冒号分割键值对
            String[] keyValue = pair.split(":", 2);
            if (keyValue.length == 2) {
                String key = keyValue[0].trim();
                String value = keyValue[1].trim();
                // 匹配 GPUMemorySize 字段（忽略大小写）
                if ("GPUMemorySize".equalsIgnoreCase(key)) {
                    try {
                        requestMemorySize = Integer.parseInt(value);
                        log.info("模型GPU内存: {} MB", requestMemorySize);
                        break; // 找到后即可退出循环
                    } catch (NumberFormatException e) {
                        log.error("解析GPUMemorySize失败，值: {}", value, e);
                        return -1;
                    }
                }
            }
        }
        if (requestMemorySize <= 0) {
            log.error("模型需求GPU内存未配置或无效");
            return -1;
        }
        return requestMemorySize;
    }
    
}
